#version 330 core
#ifdef GL_ES
precision mediump float;
precision mediump int;
#endif
uniform sampler2D Texture0;
uniform float AlphaThreshold;

uniform vec4 DrawColor;
uniform struct FogParameters
{
	vec4  FogColor;		// Fog color
	float FogStart;		// Only for linear fog
	float FogEnd;		// Oonly for linear fog
	float FogDensity;	// For exp and exp2 equation
	int FogMode; // 0 = linear, 1 = exp, 2 = exp2
} FogParams;

float getFogFactor(FogParameters Params, float FogCoord)
{
	float FogResult = 0.0;
	if(Params.FogMode == 0)
		FogResult = (Params.FogEnd-FogCoord)/(Params.FogEnd-Params.FogStart);
	else if(Params.FogMode == 1)
		FogResult = exp(-Params.FogDensity*FogCoord);
	else if(Params.FogMode == 2)
		FogResult = exp(-pow(Params.FogDensity*FogCoord, 2.0));
		
	FogResult = 1.0-clamp(FogResult, 0.0, 1.0);
	
	return FogResult;
}

in vec2 vLightMapCoords;
in vec4 vEyeSpacePos;

out vec4 FragColor; 

void main (void)  
{     
	vec4 TotalColor = texture(Texture0, vLightMapCoords); //for Lightmap
	if(TotalColor.a <= AlphaThreshold) 
        discard;

	if (DrawColor.x != 0.0 || DrawColor.y !=0.0 || DrawColor.z != 0.0 || DrawColor.w != 0.0)
		TotalColor *= DrawColor; // Add DrawColor.

	// Add distance fog
	if ((FogParams.FogMode == 0 && FogParams.FogEnd > FogParams.FogStart) || (FogParams.FogMode > 0 && FogParams.FogDensity > 0.0 ))
	{
		float FogCoord = abs(vEyeSpacePos.z/vEyeSpacePos.w);
		TotalColor = mix(TotalColor, FogParams.FogColor, getFogFactor(FogParams, FogCoord));
	}
		
	FragColor = TotalColor;             
}
