#version 330 core
#ifdef GL_ES
precision mediump float;
precision mediump int;
#endif
uniform sampler2D Texture0;	//Base Texture
uniform sampler2D Texture1;	//Lightmap
uniform sampler2D Texture2;	//DetailTexture
uniform sampler2D Texture3;	//MacroTexture
uniform sampler2D Texture4;	//BumpMap
uniform float AlphaThreshold;

uniform vec4 DrawColor;
uniform bvec4 ComplexAdds;
uniform bool bFogMap;
uniform ivec4 PolyFlags; 

uniform bool FogChanged;

in vec4 vEyeSpacePos;
in vec2 vTexCoords;
in vec2 vLightMapCoords;
in vec2 vDetailTexCoords;
in vec2 vMacroTexCoords;


layout ( location = 0, index = 0) out vec4 FragColor; 
//layout ( location = 0, index = 1) out vec4 FragColor1;

uniform struct FogParameters
{
	vec4  FogColor;		// Fog color
	float FogStart;		// Only for linear fog
	float FogEnd;		// Oonly for linear fog
	float FogDensity;	// For exp and exp2 equation
	int FogMode; // 0 = linear, 1 = exp, 2 = exp2
} FogParams;

float getFogFactor(FogParameters Params, float FogCoord)
{
	float FogResult = 0.0;
	if(Params.FogMode == 0)
		FogResult = (Params.FogEnd-FogCoord)/(Params.FogEnd-Params.FogStart);
	else if(Params.FogMode == 1)
		FogResult = exp(-Params.FogDensity*FogCoord);
	else if(Params.FogMode == 2)
		FogResult = exp(-pow(Params.FogDensity*FogCoord, 2.0));
		
	FogResult = 1.0-clamp(FogResult, 0.0, 1.0);
	
	return FogResult;
}

vec3 rgb2hsv(vec3 c)
{
	// some nice stuff from http://lolengine.net/blog/2013/07/27/rgb-to-hsv-in-glsl
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
	vec4 p = c.g < c.b ? vec4(c.bg, K.wz) : vec4(c.gb, K.xy);
    vec4 q = c.r < p.x ? vec4(p.xyw, c.r) : vec4(c.r, p.yzx);

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main (void)  
{  
	vec4 TotalColor;
	vec4 Color = texture(Texture0, vTexCoords);   
	vec4 LightColor = texture(Texture1, vLightMapCoords);  //for Lightmap
	vec4 DetailTexColor = texture(Texture2, vDetailTexCoords); // DetailTexture
	vec4 MacrotexColor = texture(Texture3, vMacroTexCoords); // MacroTexture

	TotalColor=Color;

	// DetailTextures
	if (ComplexAdds.y == true)
	{
		vec3 hsvDetailTex = rgb2hsv(DetailTexColor.rgb); // cool idea Han :)
		hsvDetailTex.b += (DetailTexColor.r - 0.1);
		hsvDetailTex = hsv2rgb(hsvDetailTex);
		DetailTexColor=vec4(hsvDetailTex,1.0);
		TotalColor*=DetailTexColor;
	}

	// MacroTextures
	if (ComplexAdds.z == true)
	{
		vec3 hsvMacroTex = rgb2hsv(MacrotexColor.rgb);
		hsvMacroTex.b += (MacrotexColor.r - 0.1);
		hsvMacroTex = hsv2rgb(hsvMacroTex);
		MacrotexColor=vec4(hsvMacroTex,1.0);
		TotalColor*=MacrotexColor;
	}

	// LightMap
	if (ComplexAdds.x == true)
	{
		TotalColor*=(LightColor*2);
		//LightMapColor = LightColor;
	}

	// BumpMap
	if (ComplexAdds.w == true)
	{
		//Extract the normal from the normal map  
		vec3 normal = normalize(texture(Texture4, vTexCoords).rgb * 2.0 - 1.0);  
  
		//Set the lightposition
		vec3 light_pos = normalize(vec3(1.0, 1.0, 1.0));  
  
		//Lighting diffuse value  
		float diffuse = max(dot(normal, light_pos), 0.0);  
  
		vec3 BumpColor = diffuse * texture(Texture0, vTexCoords).rgb; 
		TotalColor*=vec4(BumpColor,1.0);
	}
		
	TotalColor=clamp(TotalColor,0.0,1.0); //saturate.

	if(TotalColor.a <= AlphaThreshold) 
		discard;

	if (DrawColor.x != 0.0 || DrawColor.y !=0.0 || DrawColor.z != 0.0 || DrawColor.w != 0.0)
		TotalColor *= DrawColor; // Add DrawColor.

	// Add distance fog
	if ((FogParams.FogMode == 0 && FogParams.FogEnd > FogParams.FogStart) || (FogParams.FogMode > 0 && FogParams.FogDensity > 0.0 ))
	{
		float FogCoord = abs(vEyeSpacePos.z/vEyeSpacePos.w);
		TotalColor = mix(TotalColor, FogParams.FogColor, getFogFactor(FogParams, FogCoord));
	}
	//FragColor1	= mix(TotalColor,LightColor,1.0);// way to fix skybox etc??  
	FragColor	= TotalColor;  
}