#version 330 core
layout(triangles) in;
layout(triangle_fan, max_vertices=3) out;
 
in vec3 vNormal[];
in vec3 vPos[];
in vec2 vTexCoords[];
 
out vec3 gTangent;
out vec3 gBitangent;
out vec3 gNormal;
out vec3 gPos;
out vec2 gTexCoords;
 
vec3 GetTangent(vec3 A, vec3 B, vec3 C,  vec2 Auv, vec2 Buv, vec2 Cuv)
{
  float Bv_Cv = Buv.y - Cuv.y;
  if(Bv_Cv == 0.0)
    return (B-C)/(Buv.x-Cuv.x);
 
  float Quotient = (Auv.y - Cuv.y)/(Bv_Cv);
  vec3 D   = C   + (B  -C)   * Quotient;
  vec2 Duv = Cuv + (Buv-Cuv) * Quotient;
  return (D-A)/(Duv.x - Auv.x);
}
vec3 GetBitangent(vec3 A, vec3 B, vec3 C,  vec2 Auv, vec2 Buv, vec2 Cuv)
{
  return GetTangent(A, C, B,  Auv.yx, Cuv.yx, Buv.yx);
}
 
void main(void)
{
  vec3 Tangent = GetTangent(vPos[0], vPos[1], vPos[2], vTexCoords[0], vTexCoords[1], vTexCoords[2]);
  vec3 Bitangent = GetBitangent(vPos[0], vPos[1], vPos[2], vTexCoords[0], vTexCoords[1], vTexCoords[2]);
 
  for(int i=0; i<3; ++i) 
  {
    gTangent = Tangent;
    gBitangent = Bitangent;
    gTexCoord = vTexCoords[i];
    gNormal = vNormal[i];
    gPos = vPos[i];
    gl_Position = gl_in[i].gl_Position;
    EmitVertex();
  }
}