#version 330 core
#ifdef GL_ES
precision mediump float;
precision mediump int;
#endif

uniform sampler2D Texture0;
uniform float AlphaThreshold;
uniform vec4 DrawColor;
uniform int PolyFlags; 

uniform bool VertexFogColor;//VertexFog

in vec4 vEyeSpacePos;
in vec4 vLightColor;
in vec4 vFogColor; //VertexFog
in vec2 vTexCoords;

out vec4 FragColor;

//DistanceFog
uniform struct FogParameters
{
	vec4  FogColor;		// Fog color
	float FogStart;		// Only for linear fog
	float FogEnd;		// Oonly for linear fog
	float FogDensity;	// For exp and exp2 equation
	int FogMode; // 0 = linear, 1 = exp, 2 = exp2
} FogParams;

float getFogFactor(FogParameters Params, float FogCoord)
{
	float FogResult = 0.0;
	if(Params.FogMode == 0)
		FogResult = (Params.FogEnd-FogCoord)/(Params.FogEnd-Params.FogStart);
	else if(Params.FogMode == 1)
		FogResult = exp(-Params.FogDensity*FogCoord);
	else if(Params.FogMode == 2)
		FogResult = exp(-pow(Params.FogDensity*FogCoord, 2.0));
		
	FogResult = 1.0-clamp(FogResult, 0.0, 1.0);
	
	return FogResult;
}

void main(void)
{
	vec4 Color = texture(Texture0, vTexCoords); 
	vec4 TotalColor=Color;
	
	TotalColor*=vLightColor;

	if(VertexFogColor)
	{
		if (vFogColor.x != 0.0 || vFogColor.y != 0.0 || vFogColor.z != 0.0)
			TotalColor=mix(Color, vFogColor, 0.5);
	}
		

	if (DrawColor.x != 0.0 || DrawColor.y !=0.0 || DrawColor.z != 0.0 || DrawColor.w != 0.0)
		TotalColor*=DrawColor;

	// Add DistanceFog
	if ((FogParams.FogMode == 0 && FogParams.FogEnd > FogParams.FogStart) || (FogParams.FogMode > 0 && FogParams.FogDensity > 0.0 ))
	{
		float FogCoord = abs(vEyeSpacePos.z/vEyeSpacePos.w);
		TotalColor = mix(Color, FogParams.FogColor, getFogFactor(FogParams, FogCoord));
	}

	if(TotalColor.a <= AlphaThreshold) 
		discard;

	FragColor = TotalColor; 
}

// Blending translation table
/*
//PF_Modulated
//glBlendFunc( GL_DST_COLOR, GL_SRC_COLOR );
//pixel_color * gl_FragColor
	
GL_ONE 						vec4(1.0)
GL_ZERO 					vec4(0.0)
GL_SRC_COLOR 				gl_FragColor
GL_SRC_ALPHA 				vec4(gl_FragColor.a)
GL_DST_COLOR 				pixel_color
GL_DST_ALPHA 				vec4(pixel_color.a)
GL_ONE_MINUS_SRC_COLOR		vec4(1.0) - gl_FragColor
GL_ONE_MINUS_SRC_ALPHA 		vec4(1.0  - gl_FragColor.a)
GL_ONE_MINUS_DST_COLOR 		vec4(1.0) - pixel_color
GL_ONE_MINUS_DST_ALPHA 		vec4(1.0  - pixel_color.a)

if (PolyFlags & PF_Invisible)
	glBlendFunc(GL_ZERO, GL_ONE);

if (PolyFlags & PF_Translucent)
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_COLOR);

if (PolyFlags & PF_Modulated)
	glBlendFunc(GL_DST_COLOR, GL_SRC_COLOR);

if (PolyFlags & PF_AlphaBlend)
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

if (PolyFlags & PF_Highlighted)
	glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);

	//Unreal 227 PolyFlags. May requrie edits for other UE1 games. Todo: put as #include.
	const GLuint PF_Invisible			= 0x00000001u;	// Poly is invisible.
	const GLuint PF_Masked			= 0x00000002u;	// Poly should be drawn masked.
	const GLuint PF_Translucent	 	= 0x00000004u;	// Poly is transparent.
	const GLuint PF_NotSolid			= 0x00000008u;	// Poly is not solidu; doesn't block.
	const GLuint PF_Environment   	= 0x00000010u;	// Poly should be drawn environment mapped.
	const GLuint PF_Semisolid	  		= 0x00000020u;	// Poly is semi-solid = collision solidu; Csg nonsolid.
	const GLuint PF_Modulated 		= 0x00000040u;	// Modulation transparency.
	const GLuint PF_FakeBackdrop		= 0x00000080u;	// Poly looks exactly like backdrop.
	const GLuint PF_TwoSided			= 0x00000100u;	// Poly is visible from both sides.
	const GLuint PF_AutoUPan		 	= 0x00000200u;	// Automatically pans in U direction.
	const GLuint PF_AutoVPan 			= 0x00000400u;	// Automatically pans in V direction.
	const GLuint PF_NoSmooth			= 0x00000800u;	// Don't smooth textures.
	const GLuint PF_BigWavy 			= 0x00001000u;	// Poly has a big wavy pattern in it.
	const GLuint PF_SpecialPoly		= 0x00001000u;	// Game-specific poly-level render control (reuse BigWavy flag)
	const GLuint PF_SmallWavy			= 0x00002000u;	// Small wavy pattern (for water/enviro reflection).
	const GLuint PF_Flat				= 0x00004000u;	// Flat surface.
	const GLuint PF_ForceViewZone		= 0x00004000u;	// Force current iViewZone in OccludeBSP (reuse Flat flag)
	const GLuint PF_LowShadowDetail	= 0x00008000u;	// Low detail shadows.
	const GLuint PF_NoMerge			= 0x00010000u;	// Don't merge poly's nodes before lighting when rendering.
	const GLuint PF_AlphaBlend 		= 0x00020000u;	// This poly should be alpha blended
	const GLuint PF_DirtyShadows		= 0x00040000u;	// Dirty shadows.
	const GLuint PF_BrightCorners		= 0x00080000u;	// Brighten convex corners.
	const GLuint PF_SpecialLit		= 0x00100000u;	// Only speciallit lights apply to this poly.
	const GLuint PF_Gouraud			= 0x00200000u;	// Gouraud shaded.
	const GLuint PF_NoBoundRejection	= 0x00200000u;	// Disable bound rejection in OccludeBSP (reuse Gourard flag)
	const GLuint PF_Unlit				= 0x00400000u;	// Unlit.
	const GLuint PF_HighShadowDetail	= 0x00800000u;	// High detail shadows.
	const GLuint PF_Portal			= 0x04000000u;	// Portal between iZones.
	const GLuint PF_Mirrored			= 0x08000000u;	// Reflective surface.

	// Editor flags.
	const GLuint PF_Memorized     	= 0x01000000u;	// Editor: Poly is remembered.
	const GLuint PF_Selected      	= 0x02000000u;	// Editor: Poly is selected.
	const GLuint PF_Highlighted		= 0x10000000u;	// Editor: Poly is highlighted.
	const GLuint PF_FlatShaded		= 0x40000000u;	// FPoly has been split by SplitPolyWithPlane.

	// Internal.
	const GLuint PF_EdProcessed 		= 0x40000000u;	// FPoly was already processed in editorBuildFPolys.
	const GLuint PF_EdCut       		= 0x80000000u;	// FPoly has been split by SplitPolyWithPlane.
	const GLuint PF_RenderFog			= 0x40000000u;	// Render with fogmapping.
	const GLuint PF_Occlude			= 0x80000000u;	// Occludes even if const GLuint PF_NoOcclude.
	const GLuint PF_RenderHint		= 0x01000000u;   // Rendering optimization hint.

	// Combinations of flags.
	const GLuint PF_NoOcclude			= PF_Masked | PF_Translucent | PF_Invisible | PF_Modulated | PF_AlphaBlend;
	const GLuint PF_NoEdit			= PF_Memorized | PF_Selected | PF_EdProcessed | PF_NoMerge | PF_EdCut;
	const GLuint PF_NoImport			= PF_NoEdit | PF_NoMerge | PF_Memorized | PF_Selected | PF_EdProcessed | PF_EdCut;
	const GLuint PF_AddLast			= PF_Semisolid | PF_NotSolid;
	const GLuint PF_NoAddToBSP		= PF_EdCut | PF_EdProcessed | PF_Selected | PF_Memorized;
	const GLuint PF_NoShadows			= PF_Unlit | PF_Invisible | PF_FakeBackdrop;
	const GLuint PF_Transient			= PF_Highlighted;
*/